// Mesure de temperature avec SHT31
// Fichier SHT31.cpp
#include <iostream> 
#include <bcm2835.h>
#include <math.h>
using namespace std;

#define clk_div BCM2835_I2C_CLOCK_DIVIDER_2500

// g++ -o SHT31 bcm2835.cpp SHT31.cpp

void init_I2c_bcm2835()
{
	if (!bcm2835_init())
    		{
      		printf("bcm2835_init failed. Are you running as root??\n");
    		}
      	if (!bcm2835_i2c_begin())
      		{
        	printf("bcm2835_i2c_begin failed. Are you running as root??\n");
      		}
	bcm2835_i2c_setClockDivider(clk_div);			// Division de l'horloge Rpi pour obtenir une vitesse de 100KHz
}

int main(void)
{
	float mesureT, mesureRH;
	float alpha, Tr;
	int ST, SRH;
	unsigned int slave_address=0x44; // Adresse du SHT31 pour broche 2 ADDR = 0
//	unsigned int slave_address=0x45; // Adresse du SHT31 pour broche 2 ADDR = 1
	char i2cOut[3]; 	// Tableau des données I2C à sortir 
	char i2cIn[6];		// Tableau des données I2C entrées, 6 Bytes doivent être lus successivement

	init_I2c_bcm2835();
	bcm2835_i2c_setSlaveAddress(slave_address);	// Affectation de l'adresse de l'esclave 
	cout<<"Mesures sur le SHT31 : "<<endl;


	i2cOut[0] = 0x30;				// Soft reset
	i2cOut[1] = 0xA2;				// Soft reset
	bcm2835_i2c_write(i2cOut, 2); 	// 

	delay(1000);					// 1 seconde avant première lecture

	while(1)
		{
		// Mesure température
		i2cOut[0] = 0x2C; 					// Octet  "Repeatibility High", "Clock Streatching Enabled" voir doc Tableau 9 page 10/22 
		i2cOut[1] = 0x06; 					//
		bcm2835_i2c_write(i2cOut, 2); 		// Envoi Commande 
		delay(100);						// Délais permettant au capteur de température d'effectuer ses mesures
		bcm2835_i2c_read(i2cIn,6);		// Lecture des 6 octets du resultat

		// Calcul température
		//cout<<"i2cIn[0] = "<< (int)i2cIn[0]<< "      i2cIn[1] = "<< (int)i2cIn[1]<<endl;
		ST=256*(int)i2cIn[0] + (int)i2cIn[1];	// Signal Temperature
		//cout<<"ST = "<< ST<<endl;
		mesureT= (-45.00 +175.00 * ST/65535);
		cout<<"Temperature du SHT31 : "<<mesureT<<" deg."<<endl;

		// Calcul humidité relative
		//cout<<"i2cIn[3] = "<< (int)i2cIn[3]<< "      i2cIn[4] = "<< (int)i2cIn[4]<<endl;
		SRH=256*(int)i2cIn[3] + (int)i2cIn[4];	// Signal humidité relative
		//cout<<"SRH= "<< SRH<<endl;
		mesureRH= (100.00 * SRH/65535);
		cout<<"Humidité relative du SHT31 : "<<mesureRH<<" %"<<endl;

		// Calcul du point de rosée
		alpha=(17.27*mesureT)/(237.7+mesureT)+log(mesureRH/100);
		Tr = (237.7*alpha)/(17.27-alpha);
		cout<<"Point de rosée : "<<Tr<<" deg."<<endl;	
	
		cout<<endl;
		delay(1000);									// 1 seconde entre chaque mesure
   		}

	bcm2835_close();
	return 0;
}
